@@ -22,7 +22,7 @@
  * 3. This notice may not be removed or altered from any source
  *    distribution.
  *
- * $Id: CGIResponseWriter.h 2272 2006-12-17 10:21:48Z svn $
+ * $Id: CGIResponseWriter.h 2297 2006-12-24 03:13:15Z svn $
  *****************************************************************************/
 
 #ifndef CGI_RESPONSE_WRITER_H
@@ -37,7 +37,6 @@
 #include "ResponseWriter.h"
 #include "CGIResponse.h"
 
-using namespace std;
 
 /**
  * @brief コンソールにでレスポンスを書き出すクラス．
@@ -49,30 +48,30 @@ public:
     {
 
     };
-    void write(const char *str, apr_size_t length) const throw()
+    void write(const char *str, apr_size_t length) const
     {
-        cout.write(str, static_cast<streamsize>(length));
+        std::cout.write(str, static_cast<streamsize>(length));
 #ifdef DEBUG_TemplateExecutor
-        cout.flush();
+        std::cout.flush();
 #endif
     };
-    void write(const char *str) const throw()
+    void write(const char *str) const
     {
-        cout << str;
+        std::cout << str;
 #ifdef DEBUG_TemplateExecutor
-        cout.flush();
+        std::cout.flush();
 #endif
     };
-    void write(int i) const throw()
+    void write(int i) const
     {
-        cout << i;
+        std::cout << i;
 #ifdef DEBUG_TemplateExecutor
-        cout.flush();
+        std::cout.flush();
 #endif
     };
-    void finish() const throw()
+    void finish() const
     {
-        cout.flush();
+        std::cout.flush();
     };
     static int sendfile(CGIResponse::Handle *r,
                         apr_file_t *file, apr_uint64_t length)
