@@ -22,7 +22,7 @@
  * 3. This notice may not be removed or altered from any source
  *    distribution.
  *
- * $Id: ReadLocker.h 2381 2007-02-05 16:19:16Z svn $
+ * $Id: ReadLocker.h 2459 2007-06-29 16:38:34Z svn $
  *****************************************************************************/
 
 #ifndef READ_LOCKER_H
@@ -31,7 +31,6 @@
 #include "Environment.h"
 
 #include "ReadWriteLocker.h"
-#include "Macro.h"
 
 #ifdef DEBUG
 #include <iostream>
@@ -47,34 +46,48 @@ public:
     /**
      * コンストラクタです．
      *
-     * @param[out] read_count 読み出し中のスレッドの数
+     * @param[out] lock ロック変数
      */
-    ReadLocker(apr_atomic_t *read_count)
-      : ReadWriteLocker(read_count)
+    ReadLocker(apr_atomic_t *lock)
+      : ReadWriteLocker(lock)
     {
-        apr_uint32_t curr_count;
+        apr_size_t try_count;
+        apr_uint32_t status;
+        apr_uint32_t new_status;
 
+        try_count = 0;
         while (1) {
-            curr_count = apr_atomic_read(read_count_);
-            if (UNLIKELY(curr_count == WRITE_LOCKED)) {
-                yield();
-                continue;
+            try_count++;
+
+            status = apr_atomic_read(lock);
+            if (LIKELY(get_read_count(status) != WRITE_LOCKED)) {
+                new_status = make_status(get_age(status),
+                                         get_read_count(status) + 1);
+                if (apr_atomic_cas(lock_, new_status, status) == status) {
+                    status_ = new_status;
+                    return;
+                }
+            } else if (((try_count & TIMEOUT_CHECK_MASK) == 0) &&
+                       is_timeout(status)) {
+                // タイムアウトしたら age を増やす
+                new_status = make_status(get_age(status) + 1, 1);
+                if (apr_atomic_cas(lock_, new_status, status) == status) {
+                    status_ = new_status;
+                    return;
+                }
             }
 
-            if (UNLIKELY(apr_atomic_cas(read_count_, curr_count + 1, curr_count)
-                         != curr_count)) {
-                yield();
-                continue;
-            }
-            return;
+            yield();
         }
     };
     ~ReadLocker()
     {
-        (void)apr_atomic_dec(read_count_);
+        (void)apr_atomic_cas(lock_, make_status(get_age(status_),
+                                                get_read_count(status_) - 1),
+                             status_);
     };
-    static bool is_locked(apr_atomic_t *read_count) {
-        return apr_atomic_read(read_count) != 0;
+    static bool is_locked(apr_atomic_t *lock) {
+        return get_read_count(apr_atomic_read(lock)) != NOT_LOCKED;
     };
 };
 
