@@ -22,7 +22,7 @@
  * 3. This notice may not be removed or altered from any source
  *    distribution.
  *
- * $Id: UploadItemList.h 2290 2006-12-22 13:41:02Z svn $
+ * $Id: UploadItemList.h 2464 2007-06-30 01:47:25Z svn $
  *****************************************************************************/
 
 #ifndef UPLOAD_HEADER_LIST_H
@@ -89,8 +89,8 @@ public:
     };
     bool is_full(apr_uint64_t file_size) const
     {
-        return ((total_file_size_ + file_size) > max_file_size_) ||
-            (size() == max_list_size_);
+        return ((total_file_size_ + file_size) >= max_file_size_) ||
+            (size() >= max_list_size_);
     };
     /**
      * プールを使ってインスタンスを生成します．
