@@ -22,7 +22,7 @@
  * 3. This notice may not be removed or altered from any source
  *    distribution.
  *
- * $Id: UploadItemManager.h 2296 2006-12-24 02:58:57Z svn $
+ * $Id: UploadItemManager.h 2454 2007-06-29 15:22:09Z svn $
  *****************************************************************************/
 
 #ifndef UPLOAD_ITEM_MANAGER_H
@@ -75,7 +75,7 @@ public:
      * 共有メモリを使ってインスタンスを生成します．
      *
      * @param[in] shm 共有メモリ
-     * @param[in] pool プールメモリ
+     * @param[in] pool プール
      * @param[in] data_dir_path アイテムが保存されているディレクトリ
      * @param[in] file_dir_path ファイルが保存されているディレクトリ
      * @param[in] thumb_dir_path サムネイルが保存されているディレクトリ
@@ -87,8 +87,11 @@ public:
                                            const char *thumb_dir_path);
     /**
      * メモリ上にあるアイテムデータをディスクに書き出します．
+     *
+     * @param[in] pool プール
+     * @param[in] sync_limit 同期するアイテムの最大数
      */
-    void sync_all_item(apr_pool_t *pool);
+    void sync_all_item(apr_pool_t *pool, apr_size_t sync_limit=0);
     /**
      * 共有メモリを使って子プロセス用の初期化を行います．
      *
@@ -115,6 +118,11 @@ public:
     static void check_finalize_state(UploadItemManager *item_manager);
 
 private:
+    /** 書き込み時，1/SYNC_FACTOR の確率でディスクとの同期を行います． */
+    static const apr_size_t SYNC_FACTOR;
+    /** 書き込み時，一度に同期する数． */
+    static const apr_size_t SYNC_LIMIT;
+
     UploadItemManager(apr_pool_t *pool, const char *data_dir_path,
                       const char *file_dir_path, const char *thumb_dir_path);
     /**
@@ -142,6 +150,10 @@ private:
      */
     void create_item_id_index(apr_size_t *item_id,
                               apr_size_t *item_index) const;
+    /**
+     * 適度に，メモリ上にあるアイテムデータをディスクに書き出します．
+     */
+    void sync_all_item_by_catches(apr_pool_t *pool);
 
     apr_pool_t *pool_;
 
@@ -157,6 +169,8 @@ private:
     apr_time_t *mtime_;
     /** マネージャへの参照の数 (in in 共有メモリ) */
     apr_atomic_t *manager_ref_count_;
+    /** 変更カウンタ (in in 共有メモリ) */
+    apr_size_t *age_;
 };
 
 #endif
