@@ -22,7 +22,7 @@
  * 3. This notice may not be removed or altered from any source
  *    distribution.
  *
- * $Id: WriteLocker.h 2381 2007-02-05 16:19:16Z svn $
+ * $Id: WriteLocker.h 2459 2007-06-29 16:38:34Z svn $
  *****************************************************************************/
 
 #ifndef WRITE_LOCKER_H
@@ -34,7 +34,6 @@
 #include "apr_global_mutex.h"
 
 #include "ReadWriteLocker.h"
-#include "Macro.h"
 
 #ifdef DEBUG
 #include <iostream>
@@ -50,20 +49,34 @@ public:
     /**
      * コンストラクタです．
      *
-     * @param[out] read_count 読み出し中のスレッドの数
+     * @param[out] lock ロック変数
      */
-    WriteLocker(apr_atomic_t *read_count)
-      : ReadWriteLocker(read_count)
+    WriteLocker(apr_atomic_t *lock)
+      : ReadWriteLocker(lock)
     {
-        while (1) {
-            if (UNLIKELY(apr_atomic_read(read_count_) != 0)) {
-                yield();
-                continue;
-            }
+        apr_size_t try_count;
+        apr_uint32_t status;
+        apr_uint32_t new_status;
 
-            if (UNLIKELY(apr_atomic_cas(read_count_, WRITE_LOCKED, 0)
-                         == 0)) {
-                return;
+        try_count = 0;
+        while (1) {
+            try_count++;
+
+            status = apr_atomic_read(lock);
+            if (LIKELY(get_read_count(status) == NOT_LOCKED)) {
+                new_status = make_status(get_age(status), WRITE_LOCKED);
+                if (apr_atomic_cas(lock_, new_status, status) == status) {
+                    status_ = new_status;
+                    return;
+                }
+            } else if (((try_count & TIMEOUT_CHECK_MASK) == 0) &&
+                       is_timeout(status)) {
+                // タイムアウトしたら age を増やす
+                new_status = make_status(get_age(status) + 1, WRITE_LOCKED);
+                if (apr_atomic_cas(lock_, new_status, status) == status) {
+                    status_ = new_status;
+                    return;
+                }
             }
 
             yield();
@@ -71,10 +84,11 @@ public:
     };
     ~WriteLocker()
     {
-        (void)apr_atomic_cas(read_count_, 0, WRITE_LOCKED);
+        (void)apr_atomic_cas(lock_, make_status(get_age(status_), NOT_LOCKED),
+                             status_);
     };
-    static bool is_locked(apr_atomic_t *read_count) {
-        return apr_atomic_read(read_count) == ReadWriteLocker::WRITE_LOCKED;
+    static bool is_locked(apr_atomic_t *lock) {
+        return get_read_count(apr_atomic_read(lock)) == WRITE_LOCKED;
     };
 };
 
