@@ -22,7 +22,7 @@
  * 3. This notice may not be removed or altered from any source
  *    distribution.
  *
- * $Id: Auxiliary.cpp 2394 2007-03-25 10:44:25Z svn $
+ * $Id: Auxiliary.cpp 2431 2007-05-20 09:38:27Z svn $
  *****************************************************************************/
 
 #include "Environment.h"
@@ -49,7 +49,7 @@
 
 using namespace std;
 
-SOURCE_INFO_ADD("$Id: Auxiliary.cpp 2394 2007-03-25 10:44:25Z svn $");
+SOURCE_INFO_ADD("$Id: Auxiliary.cpp 2431 2007-05-20 09:38:27Z svn $");
 
 void init_atomic(apr_pool_t *pool)
 {
@@ -360,34 +360,41 @@ const char *strnchr(const char *s, size_t length, int c)
     unsigned long word_char;
     unsigned long magic;
     unsigned long cmask;
-    int count;
+    apr_size_t count;
+    apr_size_t tail_count;
 
-    count = static_cast<int>(length);
+    pos = const_cast<char *>(s);
+    count = static_cast<apr_size_t>(length);
 
     // ワード境界まではバイト単位でチェック．
-    for (pos = const_cast<char *>(s);
-         (reinterpret_cast<apr_uint64_t>(pos) & (sizeof(long)-1)) != 0;
-         pos++) {
-        if (*pos == c) {
-            return pos;
-        } else if (*pos == '\0') {
-            return NULL;
-        }
-
-        if (--count == 0) {
-            return NULL;
+    tail_count = static_cast<apr_size_t>(reinterpret_cast<apr_uint64_t>(pos) &
+                                         (sizeof(long)-1));
+    if (tail_count) {
+        tail_count = sizeof(long) - tail_count;
+        while (tail_count--) {
+            if (count-- == 0) {
+                return NULL;
+            }
+            if (*pos == c) {
+                return pos;
+            } else if (*pos++ == '\0') {
+                return NULL;
+            }
         }
     }
 
     word_pos = reinterpret_cast<const unsigned long *>(pos);
 
     if (sizeof(long) == 4) {
-        magic = 0x7efefeffL;
+        magic = 0x7EFEFEFFL;
+        tail_count = count & 0x3;
     } else if (sizeof(long) == 8) {
-        magic = ((0x7efefefeL << 16) << 16) | 0xfefefeffL;
+        magic = ((0x7EFEFEFEL << 16) << 16) | 0xFEFEFEFFL;
+        tail_count = count & 0x7;
     } else {
         abort();
     }
+    count -= tail_count;
 
     cmask = c | (c << 8);
     cmask |= cmask << 16;
@@ -396,88 +403,39 @@ const char *strnchr(const char *s, size_t length, int c)
     }
 
     // ワード単位でチェック．
-    do {
+    for (; count > 0; count -= sizeof(long)) {
         word_char = *word_pos++;
 
-        // Valgrind で
-        // 「Conditional jump or move depends on uninitialised value(s)」
-        // が発生する場合があるけど問題ないです
         if (LIKELY((((((word_char) +
                        magic) ^ ~(word_char)) & ~magic) == 0) &&
                    (((((word_char ^ cmask) +
                        magic) ^ ~(word_char ^ cmask)) & ~magic) == 0))) {
-            count -= sizeof(long);
             continue;
         }
 
         pos = reinterpret_cast<const char *>(word_pos - 1);
 
+        for (apr_size_t i = sizeof(long); i != 0; i--) {
+            if (*pos == c) {
+                return pos;
+            } else if (*pos++ == '\0') {
+                return NULL;
+            }
+        }
+    };
+
+    // Valgrind で
+    // 「Conditional jump or move depends on uninitialised value(s)」
+    // が発生するのが嫌な感じなので，末尾の半端な部分については 1 文字
+    // ずつチェックする．
+    pos = reinterpret_cast<const char *>(word_pos);
+    while (tail_count--) {
         if (*pos == c) {
             return pos;
-        } else if (*pos == '\0') {
-            return NULL;
-        } else if (--count == 0) {
+        } else if (*pos++ == '\0') {
             return NULL;
         }
-
-        if (*(++pos) == c) {
-            return pos;
-        } else if (*pos == '\0') {
-            return NULL;
-        } else if (--count == 0) {
-            return NULL;
-        }
-
-        if (*(++pos) == c) {
-            return pos;
-        } else if (*pos == '\0') {
-            return NULL;
-        } else if (--count == 0) {
-            return NULL;
-        }
-
-        if (*(++pos) == c) {
-            return pos;
-        } else if (*pos == '\0') {
-            return NULL;
-        } else if (--count == 0) {
-            return NULL;
-        }
-
-        if (sizeof(long) == 8) {
-            if (*(++pos) == c) {
-                return pos;
-            } else if (*pos == '\0') {
-                return NULL;
-            } else if (--count == 0) {
-                return NULL;
-            }
-
-            if (*(++pos) == c) {
-                return pos;
-            } else if (*pos == '\0') {
-                return NULL;
-            } else if (--count == 0) {
-                return NULL;
-            }
-
-            if (*(++pos) == c) {
-                return pos;
-            } else if (*pos == '\0') {
-                return NULL;
-            } else if (--count == 0) {
-                return NULL;
-            }
-
-            if (*(++pos) == c) {
-                return pos;
-            } else if (*pos == '\0') {
-                return NULL;
-            } else if (--count == 0) {
-                return NULL;
-            }
-        }
-    } while (count > 0);
+    }
 
     return NULL;
 }
