@@ -22,7 +22,7 @@
  * 3. This notice may not be removed or altered from any source
  *    distribution.
  *
- * $Id: DownloadFlowController.cpp 2374 2007-01-23 13:42:21Z svn $
+ * $Id: DownloadFlowController.cpp 2381 2007-02-05 16:19:16Z svn $
  *****************************************************************************/
 
 #include "Environment.h"
@@ -35,7 +35,7 @@
 #include "WriteLocker.h"
 #include "SourceInfo.h"
 
-SOURCE_INFO_ADD("$Id: DownloadFlowController.cpp 2374 2007-01-23 13:42:21Z svn $");
+SOURCE_INFO_ADD("$Id: DownloadFlowController.cpp 2381 2007-02-05 16:19:16Z svn $");
 
 /******************************************************************************
  * public メソッド
@@ -44,7 +44,7 @@ DownloadFlowController::DownloadFlowController()
   : downloader_count_(0)
 {
     memset(downloader_list_, 0, sizeof(downloader_list_));
-    (void)apr_atomic_set(&lock_, ReadWriteLocker::NOT_LOCKER);
+    (void)apr_atomic_set(&lock_, ReadWriteLocker::NOT_LOCKED);
 }
 
 bool DownloadFlowController::add_session(apr_sockaddr_t *sockaddr)
@@ -84,8 +84,9 @@ void DownloadFlowController::remove_session(apr_sockaddr_t *sockaddr)
         THROW(MESSAGE_BUG_FOUND);
     }
 
-    // downloader が NULL ではない場合，downloader->session_count は 0
-    // より大きい
+    if (downloader->session_count == 0) {
+        THROW(MESSAGE_BUG_FOUND);
+    }
     downloader->session_count--;
 
     if (downloader_count_ == 0) {
