@@ -22,7 +22,7 @@
  * 3. This notice may not be removed or altered from any source
  *    distribution.
  *
- * $Id: UploadItemManager.cpp 2367 2007-01-08 13:42:02Z svn $
+ * $Id: UploadItemManager.cpp 2458 2007-06-29 16:15:56Z svn $
  *****************************************************************************/
 
 #include "Environment.h"
@@ -49,7 +49,7 @@
 
 using namespace std;
 
-SOURCE_INFO_ADD("$Id: UploadItemManager.cpp 2367 2007-01-08 13:42:02Z svn $");
+SOURCE_INFO_ADD("$Id: UploadItemManager.cpp 2458 2007-06-29 16:15:56Z svn $");
 
 #ifdef DEBUG
 #include <iostream>
@@ -69,6 +69,10 @@ SOURCE_INFO_ADD("$Id: UploadItemManager.cpp 2367 2007-01-08 13:42:02Z svn $");
 #define REQUIRE_MANAGER_LOCKED()
 #endif
 
+const apr_size_t UploadItemManager::SYNC_FACTOR = 32;
+const apr_size_t UploadItemManager::SYNC_LIMIT  = 100;
+
+
 /******************************************************************************
  * public メソッド
  *****************************************************************************/
@@ -122,7 +126,8 @@ apr_size_t UploadItemManager::add(apr_pool_t *pool,
     item_list_->add(uitem);
 
     *mtime_ = current_time;
-    sync_all_item(pool);
+
+    sync_all_item_by_catches(pool);
 
     return item_id;
 }
@@ -139,18 +144,25 @@ void UploadItemManager::remove(apr_pool_t *pool, UploadItem *uitem)
     thumbnail_list_->remove(uitem->get_id());
 
     *mtime_ = apr_time_now();
-    sync_all_item(pool);
+
+    sync_all_item_by_catches(pool);
 }
 
-void UploadItemManager::sync_all_item(apr_pool_t *pool)
+void UploadItemManager::sync_all_item(apr_pool_t *pool, apr_size_t sync_limit)
 {
     UploadItem::header_t *header;
+    apr_size_t synced_count;
 
+    synced_count = 0;
     for (apr_size_t i = 0; i < item_list_->size(); i++) {
         header = item_list_->get_by_index(i);
 
         if (header->is_need_sync) {
             item_writer_.write_data(pool, header);
+
+            if (++synced_count == sync_limit) {
+                break;
+            }
         }
     }
 }
@@ -229,7 +241,7 @@ UploadItemManager *UploadItemManager::child_init(apr_shm_t *shm,
 apr_size_t UploadItemManager::get_memory_size()
 {
     return sizeof(UploadItemManager) + sizeof(apr_time_t) +
-        sizeof(apr_atomic_t);
+        sizeof(apr_atomic_t) + sizeof(apr_size_t);
 }
 
 void UploadItemManager::check_finalize_state(UploadItemManager *item_manager)
@@ -274,11 +286,12 @@ void UploadItemManager::init_shm()
 {
     mtime_ = AS_TIME(this + 1);
     manager_ref_count_ = AS_ATOMIC(mtime_ + 1);
+    age_ = AS_SIZE(manager_ref_count_ + 1);
 
 #ifdef __INTEL_COMPILER
 #pragma warning(disable:1684)
 #endif
-    ASSERT_EQUAL(AS_CHAR(manager_ref_count_ + 1),
+    ASSERT_EQUAL(AS_CHAR(age_ + 1),
                  AS_CHAR(mtime_) +
                  (get_memory_size() - sizeof(UploadItemManager)));
 #ifdef __INTEL_COMPILER
@@ -310,6 +323,15 @@ void UploadItemManager::create_item_id_index(apr_size_t *item_id,
     *item_index = header->index + 1;
 }
 
+void UploadItemManager::sync_all_item_by_catches(apr_pool_t *pool)
+{
+    if ((*age_  % SYNC_FACTOR) == 0) {
+        sync_all_item(pool, SYNC_LIMIT);
+    }
+
+    *age_ += 1;
+}
+
 // Local Variables:
 // mode: c++
 // coding: utf-8-dos
