@@ -26,8 +26,8 @@ run ./util/win_build.pl
 run make -f GNUmakefile.dist dist-sf
 run make -f GNUmakefile.dist dist-win-sf
 
-run cp mod_uploader-ap2.0.so /home/apache/${PACKAGE}/
-run cp mod_uploader-ap2.2.so /home/apache/${PACKAGE}/
+run cp src/mod_uploader-ap2.0.so /home/apache/${PACKAGE}/
+run cp src/mod_uploader-ap2.2.so /home/apache/${PACKAGE}/
 
 (
     run cd doc
